
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.037.
use strict;
use warnings;

use Module::Build 0.30;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.30"
  },
  "configure_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "Module::Build" => "0.30"
  },
  "dist_abstract" => "Greenland postal code validator and container",
  "dist_author" => [
    "Jonas B. Nielsen <jonasbn\@cpan.org>"
  ],
  "dist_name" => "Business-GL-Postalcode",
  "dist_version" => "0.04",
  "license" => "artistic_2",
  "module_name" => "Business::GL::Postalcode",
  "recommends" => {},
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Data::Handle" => 0,
    "Exporter" => 0,
    "List::Util" => 0,
    "Params::Validate" => 0,
    "perl" => "5.010"
  },
  "script_files" => [],
  "test_requires" => {
    "Data::Dumper" => 0,
    "Env" => 0,
    "File::Spec" => 0,
    "IO::Handle" => 0,
    "IPC::Open3" => 0,
    "Pod::Coverage::TrustPod" => 0,
    "Test::Class" => 0,
    "Test::Kwalitee" => "1.21",
    "Test::More" => "0.88",
    "Test::Pod" => "1.41",
    "Test::Pod::Coverage" => "1.08"
  }
);


my %fallback_build_requires = (
  "Data::Dumper" => 0,
  "Env" => 0,
  "File::Spec" => 0,
  "IO::Handle" => 0,
  "IPC::Open3" => 0,
  "Module::Build" => "0.30",
  "Pod::Coverage::TrustPod" => 0,
  "Test::Class" => 0,
  "Test::Kwalitee" => "1.21",
  "Test::More" => "0.88",
  "Test::Pod" => "1.41",
  "Test::Pod::Coverage" => "1.08"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);

$build->create_build_script;
